import React from "react";
import { NativeSyntheticEvent, NativeScrollEvent, StyleProp, ViewStyle, ScrollViewProps, Animated, RefreshControlPropsIOS, RefreshControlPropsAndroid, ScrollView } from "react-native";
export interface WaterfallProps<TItem> extends ScrollViewProps {
    columnNum: number;
    columnGap?: number;
    itemInfos: TItem[];
    bufferAmount?: number;
    infiniteThreshold?: number;
    heightGetter: (width: number, index: number) => number;
    renderItem: (itemInfo: TItem, width: number, height: number, index: number) => React.ReactNode;
    renderLoadMore?: (isLoading: boolean) => React.ReactNode;
    HeaderComponent?: React.ComponentType<any> | React.ReactElement;
    FooterComponent?: React.ComponentType<any> | React.ReactElement;
    onInfinite?: () => Promise<void>;
    onRefresh?: () => Promise<void>;
    onScroll?: (event: NativeSyntheticEvent<NativeScrollEvent>) => void;
    refreshControlProps?: RefreshControlPropsIOS & RefreshControlPropsAndroid;
    style?: StyleProp<ViewStyle>;
    containerStyle?: StyleProp<ViewStyle>;
}
declare type State = {
    columnWidth: number;
    offset: number;
    refreshing: boolean;
    loading: boolean;
};
export default class Waterfall<TItem = any> extends React.Component<WaterfallProps<TItem>, State> {
    static defaultProps: {
        columnGap: number;
        bufferAmount: number;
        infiniteThreshold: number;
        renderLoadMore: (loading: boolean) => JSX.Element;
    };
    scrollViewRef: React.RefObject<ScrollView>;
    scrollOffset: Animated.Value;
    itemsRunwayOffset: Animated.Value;
    scrollHeight?: number;
    itemsRunwayWidth: number;
    lastMeasuredIndex: number;
    itemPositions: Array<{
        offsetLeft: number;
        offsetTop: number;
    }>;
    itemOffsetTops: Array<number>;
    constructor(props: WaterfallProps<TItem>);
    getColumnWidth: () => number;
    reset: () => void;
    scrollTo: (y?: number | {
        x?: number | undefined;
        y?: number | undefined;
        animated?: boolean | undefined;
    } | undefined, x?: number | undefined, animated?: boolean | undefined) => void;
    private onScroll;
    private onRefresh;
    private onInfinite;
    private onScrollLayout;
    private onItemsRunwayLayout;
    private getOffsetColumn;
    getLowestOffsetColumn(): [number, number];
    getHighestOffsetColumn(): [number, number];
    evaluateVisibleRange(): number[];
    private binarySearch;
    private exponentialSearch;
    getPositionForIndex(index: number): {
        offsetLeft: number;
        offsetTop: number;
    };
    render(): JSX.Element;
}
export {};
