var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React from "react";
import { View, Text, Animated, StyleSheet, RefreshControl, ActivityIndicator, ScrollView, } from "react-native";
export default class Waterfall extends React.Component {
    constructor(props) {
        super(props);
        this.scrollViewRef = React.createRef();
        this.scrollOffset = new Animated.Value(0);
        this.itemsRunwayOffset = new Animated.Value(0);
        this.itemsRunwayWidth = 0;
        this.lastMeasuredIndex = -1;
        this.itemPositions = [];
        this.itemOffsetTops = [];
        this.getColumnWidth = () => {
            return this.state.columnWidth;
        };
        this.reset = () => {
            this.lastMeasuredIndex = -1;
            this.itemPositions = [];
            this.itemOffsetTops = Array(this.props.columnNum).fill(0);
        };
        this.scrollTo = (y, x, animated) => {
            var _a;
            (_a = this.scrollViewRef.current) === null || _a === void 0 ? void 0 : _a.scrollTo(y, x, animated);
        };
        this.onScroll = ({ nativeEvent: { contentOffset: { y }, layoutMeasurement: { height }, contentSize: { height: contentHeight }, }, }) => {
            this.setState({ offset: y });
            if (y + height >= contentHeight - this.props.infiniteThreshold &&
                this.lastMeasuredIndex === this.props.itemInfos.length - 1 &&
                !this.state.refreshing &&
                !this.state.loading &&
                this.props.onInfinite) {
                this.onInfinite();
            }
        };
        this.onRefresh = () => {
            this.setState({ refreshing: true });
            return this.props.onRefresh().finally(() => {
                this.reset();
                this.setState({ refreshing: false });
            });
        };
        this.onInfinite = () => {
            this.setState({ loading: true });
            return this.props.onInfinite().finally(() => {
                this.setState({ loading: false });
            });
        };
        this.onScrollLayout = ({ nativeEvent: { layout: { height }, }, }) => {
            if (this.scrollHeight !== height) {
                this.scrollHeight = height;
            }
        };
        this.onItemsRunwayLayout = ({ nativeEvent: { layout: { width }, }, }) => {
            if (this.itemsRunwayWidth !== width) {
                const { columnNum, columnGap } = this.props;
                this.itemsRunwayWidth = width;
                const newColumnWidth = (width - (columnNum - 1) * columnGap) / columnNum;
                this.reset();
                this.setState({
                    columnWidth: newColumnWidth,
                });
            }
        };
        this.itemOffsetTops = Array(this.props.columnNum).fill(0);
        const state = {
            offset: 0,
            columnWidth: 0,
            refreshing: false,
            loading: false,
        };
        this.state = state;
    }
    getOffsetColumn(predicate) {
        let index;
        let value;
        this.itemOffsetTops.forEach((item, i) => {
            if ((index === undefined && value === undefined) ||
                predicate(item, value)) {
                value = item;
                index = i;
            }
        });
        return [index || 0, value || 0];
    }
    getLowestOffsetColumn() {
        return this.getOffsetColumn((p, c) => p < c);
    }
    getHighestOffsetColumn() {
        return this.getOffsetColumn((p, c) => p > c);
    }
    evaluateVisibleRange() {
        let { offset, columnWidth } = this.state;
        const { itemInfos, bufferAmount, heightGetter } = this.props;
        const maxOffset = offset + this.scrollHeight * 1.5;
        const itemCount = itemInfos.length;
        let start = 0;
        const lastMeasuredOffset = this.lastMeasuredIndex >= 0
            ? this.getPositionForIndex(this.lastMeasuredIndex).offsetTop
            : 0;
        const lastMeasuredIndex = Math.max(0, this.lastMeasuredIndex);
        const compare = (i) => this.getPositionForIndex(i).offsetTop <= offset;
        if (lastMeasuredOffset >= offset) {
            start = this.binarySearch({
                minIndex: 0,
                maxIndex: lastMeasuredIndex,
                compare,
            });
        }
        else {
            start = this.exponentialSearch({
                arrayLength: itemCount,
                index: lastMeasuredIndex,
                compare,
            });
        }
        offset =
            this.getPositionForIndex(start).offsetTop +
                heightGetter(columnWidth, start);
        let end = start;
        while (offset < maxOffset && end < itemCount - 1) {
            end++;
            offset = this.getPositionForIndex(end).offsetTop;
        }
        if (bufferAmount) {
            start = Math.max(0, start - bufferAmount);
            end = Math.min(end + bufferAmount, itemCount - 1);
        }
        return [start, end];
    }
    binarySearch({ minIndex, maxIndex, compare, }) {
        while (maxIndex >= minIndex) {
            const middle = minIndex + Math.floor((maxIndex - minIndex) / 2);
            if (compare(middle)) {
                minIndex = middle + 1;
            }
            else {
                maxIndex = middle - 1;
            }
        }
        if (minIndex > 0) {
            return minIndex - 1;
        }
        //not found :)
        return 0;
    }
    exponentialSearch({ arrayLength, index, compare, }) {
        let interval = 1;
        while (index < arrayLength && compare(index)) {
            index += interval;
            interval *= 2;
        }
        return this.binarySearch({
            minIndex: Math.min(index, arrayLength - 1),
            maxIndex: Math.floor(index / 2),
            compare,
        });
    }
    getPositionForIndex(index) {
        const { columnGap, heightGetter } = this.props;
        const { columnWidth } = this.state;
        if (index > this.lastMeasuredIndex) {
            for (let i = this.lastMeasuredIndex + 1; i <= index; i++) {
                const [columnIndex, columnOffset] = this.getLowestOffsetColumn();
                this.itemPositions[i] = {
                    offsetLeft: (columnWidth + columnGap) * columnIndex,
                    offsetTop: columnOffset,
                };
                this.itemOffsetTops[columnIndex] =
                    columnOffset + heightGetter(columnWidth, i);
                this.lastMeasuredIndex = i;
            }
        }
        return this.itemPositions[index];
    }
    render() {
        const _a = this.props, { renderItem, renderLoadMore, HeaderComponent, FooterComponent, itemInfos, heightGetter, onScroll, onRefresh, onInfinite, style, containerStyle, refreshControlProps, 
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        children } = _a, rest = __rest(_a, ["renderItem", "renderLoadMore", "HeaderComponent", "FooterComponent", "itemInfos", "heightGetter", "onScroll", "onRefresh", "onInfinite", "style", "containerStyle", "refreshControlProps", "children"]);
        const { columnWidth, loading, refreshing } = this.state;
        const items = [];
        if (columnWidth && itemInfos.length) {
            const [start, end] = this.evaluateVisibleRange();
            for (let i = start; i <= end; i++) {
                const position = this.getPositionForIndex(i);
                items.push(<View key={i} 
                // eslint-disable-next-line react-native/no-inline-styles
                style={{
                    position: "absolute",
                    top: position.offsetTop,
                    left: position.offsetLeft,
                    width: columnWidth,
                    height: heightGetter(columnWidth, i),
                }}>
            {renderItem(itemInfos[i], columnWidth, heightGetter(columnWidth, i), i)}
          </View>);
            }
            const [, runwayOffset] = this.getHighestOffsetColumn();
            this.itemsRunwayOffset.setValue(runwayOffset);
        }
        return (<View style={[styles.container]}>
        <ScrollView ref={this.scrollViewRef} style={style} bounces={false} onScroll={(e) => {
            this.onScroll(e);
            onScroll === null || onScroll === void 0 ? void 0 : onScroll.call(undefined, e);
        }} onLayout={this.onScrollLayout} refreshControl={onRefresh ? (<RefreshControl refreshing={refreshing} onRefresh={this.onRefresh} {...refreshControlProps}/>) : undefined} scrollEventThrottle={20} {...rest}>
          {HeaderComponent}
          <Animated.View style={[styles.container, containerStyle]}>
            <Animated.View style={[{ height: this.itemsRunwayOffset }, containerStyle]} onLayout={this.onItemsRunwayLayout}>
              {items}
            </Animated.View>
            {!!onInfinite && (renderLoadMore === null || renderLoadMore === void 0 ? void 0 : renderLoadMore.call(undefined, loading))}
          </Animated.View>
          {FooterComponent}
        </ScrollView>
      </View>);
    }
}
Waterfall.defaultProps = {
    columnGap: 0,
    bufferAmount: 10,
    infiniteThreshold: 50,
    renderLoadMore: (loading) => (<View style={[
        styles.loadingMoreBox,
        // eslint-disable-next-line react-native/no-inline-styles
        loading ? { opacity: 1 } : { opacity: 0 },
    ]}>
        <ActivityIndicator style={styles.loadingMoreIndicator} size="small" color="black"/>
        <Text>{"loading"}</Text>
      </View>),
};
const styles = StyleSheet.create({
    container: { flex: 1 },
    loadingMoreBox: {
        flexDirection: "row",
        justifyContent: "center",
        padding: 20,
    },
    loadingMoreIndicator: {
        marginRight: 10,
    },
});
//# sourceMappingURL=waterfall.js.map